import { MetadataJson, programs } from "@renec-foundation/metaplex-js";
import { Connection, PublicKey } from "@solana/web3.js";
import { getAccount } from "spl-token";

const IMAGE_MIME_TYPE_REGEX = /data:image\/(svg\+xml|png|jpeg|gif)/g;

const Metadata = programs.metadata.Metadata;

export const getMetaDataJSON = async (
  id: string,
  metadata: programs.metadata.MetadataData
): Promise<MetadataJson | undefined> => {
  return new Promise(resolve => {
    const uri = metadata.data.uri;
    if (!uri) return resolve(undefined);

    const processJson = (extended: any) => {
      if (!extended || (!extended.image && extended?.properties?.files?.length === 0)) {
        return;
      }
      if (extended?.image) {
        extended.image =
          extended.image.startsWith('http') || IMAGE_MIME_TYPE_REGEX.test(extended.image)
            ? extended.image
            : `${metadata.data.uri}/${extended.image}`;
      }

      return extended;
    };

    try {
      fetch(uri)
        .then(async _ => {
          try {
            const data = await _.json();
            resolve(processJson(data));
          } catch {
            resolve(undefined);
          }
        })
        .catch(() => {
          resolve(undefined);
        });
    } catch (ex) {
      console.error(ex);
      resolve(undefined);
    }
  });
};


export const fetchNFTDetail = async (nftAddress: string, connection: Connection) => {
  try {
    let nftData = null
    const pda = await Metadata.getPDA(new PublicKey(nftAddress))
    const metadata = await Metadata.load(connection, pda)
    if (metadata) {
      const metadataJSON = await getMetaDataJSON(nftAddress, metadata.data);
      nftData = {
        json: metadataJSON,
        metadata: metadata.data,
      };
    }
    return nftData
  }
  catch (error) {
    return null
  }
}

export const getAccountInfo = async (address: PublicKey, connection: Connection) => {
  try {
    return await getAccount(connection, address)
  }
  catch (err) {
    return null
  }
}